import {
  type Product,
  type CartItem,
  type Newsletter,
  type Contact,
  type InsertProduct,
  type InsertCartItem,
  type InsertNewsletter,
  type InsertContact,
} from "@shared/schema";

export interface IStorage {
  // Products
  getProducts(): Promise<Product[]>;
  getProduct(id: number): Promise<Product | undefined>;
  createProduct(product: InsertProduct): Promise<Product>;

  // Cart
  getCartItems(sessionId: string): Promise<CartItem[]>;
  addCartItem(item: InsertCartItem): Promise<CartItem>;
  removeCartItem(id: number): Promise<void>;
  updateCartItemQuantity(id: number, quantity: number): Promise<CartItem>;

  // Newsletter
  addNewsletterSubscription(sub: InsertNewsletter): Promise<Newsletter>;

  // Contact
  createContact(contact: InsertContact): Promise<Contact>;
}

export class MemStorage implements IStorage {
  private products: Map<number, Product>;
  private cartItems: Map<number, CartItem>;
  private newsletters: Map<number, Newsletter>;
  private contacts: Map<number, Contact>;
  private currentIds: {
    products: number;
    cartItems: number;
    newsletters: number;
    contacts: number;
  };

  constructor() {
    this.products = new Map();
    this.cartItems = new Map();
    this.newsletters = new Map();
    this.contacts = new Map();
    this.currentIds = {
      products: 1,
      cartItems: 1,
      newsletters: 1,
      contacts: 1,
    };

    const sampleProducts: InsertProduct[] = [
      {
        name: "Classic Tote",
        description: "Elegant everyday leather tote bag with spacious interior and magnetic closure. Perfect for work or weekend outings.",
        price: "299.99",
        images: [
          "https://images.unsplash.com/photo-1591561954557-26941169b49e",
          "https://images.unsplash.com/photo-1584917865442-de89df76afd3"
        ],
        category: "Totes",
        inStock: 10
      },
      {
        name: "Executive Briefcase",
        description: "Professional leather briefcase with multiple compartments and laptop sleeve. Ideal for business professionals.",
        price: "399.99",
        images: [
          "https://images.unsplash.com/photo-1524498250077-390f9e378fc0",
          "https://images.unsplash.com/photo-1553062407-98eeb64c6a62"
        ],
        category: "Briefcases",
        inStock: 5
      },
      {
        name: "Mini Crossbody",
        description: "Compact crossbody bag perfect for essentials. Features adjustable strap and secure zip closure.",
        price: "199.99",
        images: [
          "https://images.unsplash.com/photo-1598532163257-ae3c6b2524b6",
          "https://images.unsplash.com/photo-1579642761360-eabd1d69c8fc"
        ],
        category: "Crossbody",
        inStock: 15
      },
      {
        name: "Evening Clutch",
        description: "Elegant leather clutch with gold-tone hardware. Perfect for formal events and special occasions.",
        price: "149.99",
        images: [
          "https://images.unsplash.com/photo-1566150905458-1bf1fc113f0d",
          "https://images.unsplash.com/photo-1575891386914-5a68a4c55943"
        ],
        category: "Clutches",
        inStock: 8
      },
      {
        name: "Vintage Satchel",
        description: "Classic leather satchel with buckle details and comfortable shoulder strap. Timeless design meets modern functionality.",
        price: "279.99",
        images: [
          "https://images.unsplash.com/photo-1590874103328-eac38a683ce7",
          "https://images.unsplash.com/photo-1628149455678-16f37bc392f4"
        ],
        category: "Satchels",
        inStock: 12
      },
      {
        name: "Leather Backpack",
        description: "Sophisticated leather backpack with padded straps and multiple pockets. Perfect for the modern professional.",
        price: "329.99",
        images: [
          "https://images.unsplash.com/photo-1622560480605-d83c853bc5c3",
          "https://images.unsplash.com/photo-1548036328-c9fa89d128fa"
        ],
        category: "Backpacks",
        inStock: 7
      },
      {
        name: "Weekend Duffle",
        description: "Spacious leather duffle bag with shoulder strap. Ideal for short trips and weekend getaways.",
        price: "449.99",
        images: [
          "https://images.unsplash.com/photo-1520939817895-060bdaf4fe1b",
          "https://images.unsplash.com/photo-1553062407-98eeb64c6a62"
        ],
        category: "Travel",
        inStock: 6
      },
      {
        name: "Hobo Shoulder Bag",
        description: "Casual yet elegant hobo bag with soft leather construction. Features a roomy interior and magnetic closure.",
        price: "259.99",
        images: [
          "https://images.unsplash.com/photo-1605733513597-a8f8341084e6",
          "https://images.unsplash.com/photo-1591561954555-607968c989ab"
        ],
        category: "Shoulder Bags",
        inStock: 9
      },
      {
        name: "Work Tote XL",
        description: "Oversized leather tote with dedicated laptop compartment and organization pockets. Perfect for busy professionals.",
        price: "379.99",
        images: [
          "https://images.unsplash.com/photo-1590739293931-a0975667f140",
          "https://images.unsplash.com/photo-1524498250077-390f9e378fc0"
        ],
        category: "Totes",
        inStock: 4
      },
      {
        name: "Compact Wallet Bag",
        description: "Mini bag that doubles as a wallet with card slots and removable chain strap.",
        price: "129.99",
        images: [
          "https://images.unsplash.com/photo-1606522754091-a3ea32ec1b2b",
          "https://images.unsplash.com/photo-1584917865442-de89df76afd3"
        ],
        category: "Crossbody",
        inStock: 20
      }
    ];

    // Generate additional products programmatically
    const categories = ["Totes", "Briefcases", "Crossbody", "Clutches", "Satchels", "Backpacks", "Travel", "Shoulder Bags"];
    const adjectives = ["Luxury", "Premium", "Designer", "Artisan", "Handcrafted", "Elite", "Signature", "Exclusive"];
    const styles = ["Classic", "Modern", "Vintage", "Contemporary", "Traditional", "Elegant", "Minimalist", "Sophisticated"];

    // Collection of diverse bag images
    const categoryImages = {
      Totes: [
        ["https://images.unsplash.com/photo-1590739293931-a0975667f140", "https://images.unsplash.com/photo-1584917865442-de89df76afd3"],
        ["https://images.unsplash.com/photo-1591561954557-26941169b49e", "https://images.unsplash.com/photo-1575201647632-45fae95c9ce4"],
        ["https://images.unsplash.com/photo-1590739293931-a0975667f140", "https://images.unsplash.com/photo-1606522754091-a3ea32ec1b2b"]
      ],
      Briefcases: [
        ["https://images.unsplash.com/photo-1524498250077-390f9e378fc0", "https://images.unsplash.com/photo-1553062407-98eeb64c6a62"],
        ["https://images.unsplash.com/photo-1549439602-43ebca2327af", "https://images.unsplash.com/photo-1523293915678-d126868e96f1"],
        ["https://images.unsplash.com/photo-1622560480605-d83c853bc5c3", "https://images.unsplash.com/photo-1548036328-c9fa89d128fa"]
      ],
      Crossbody: [
        ["https://images.unsplash.com/photo-1598532163257-ae3c6b2524b6", "https://images.unsplash.com/photo-1579642761360-eabd1d69c8fc"],
        ["https://images.unsplash.com/photo-1606522754091-a3ea32ec1b2b", "https://images.unsplash.com/photo-1584917865442-de89df76afd3"],
        ["https://images.unsplash.com/photo-1591561954557-26941169b49e", "https://images.unsplash.com/photo-1575201647632-45fae95c9ce4"]
      ],
      Clutches: [
        ["https://images.unsplash.com/photo-1566150905458-1bf1fc113f0d", "https://images.unsplash.com/photo-1575891386914-5a68a4c55943"],
        ["https://images.unsplash.com/photo-1584917865442-de89df76afd3", "https://images.unsplash.com/photo-1591561954557-26941169b49e"],
        ["https://images.unsplash.com/photo-1605733513597-a8f8341084e6", "https://images.unsplash.com/photo-1591561954555-607968c989ab"]
      ],
      Satchels: [
        ["https://images.unsplash.com/photo-1590874103328-eac38a683ce7", "https://images.unsplash.com/photo-1628149455678-16f37bc392f4"],
        ["https://images.unsplash.com/photo-1524498250077-390f9e378fc0", "https://images.unsplash.com/photo-1553062407-98eeb64c6a62"],
        ["https://images.unsplash.com/photo-1622560480605-d83c853bc5c3", "https://images.unsplash.com/photo-1548036328-c9fa89d128fa"]
      ],
      Backpacks: [
        ["https://images.unsplash.com/photo-1622560480605-d83c853bc5c3", "https://images.unsplash.com/photo-1548036328-c9fa89d128fa"],
        ["https://images.unsplash.com/photo-1585488433867-8eb1de13cbe1", "https://images.unsplash.com/photo-1553062407-98eeb64c6a62"],
        ["https://images.unsplash.com/photo-1491637639811-60e2756cc1c7", "https://images.unsplash.com/photo-1494726161322-5360d4d0eeae"]
      ],
      Travel: [
        ["https://images.unsplash.com/photo-1520939817895-060bdaf4fe1b", "https://images.unsplash.com/photo-1547949003-9792a18a2601"],
        ["https://images.unsplash.com/photo-1590874103328-eac38a683ce7", "https://images.unsplash.com/photo-1596331294487-75a7df9c5ec4"],
        ["https://images.unsplash.com/photo-1578916171728-46686eac8d58", "https://images.unsplash.com/photo-1575891386914-5a68a4c55943"]
      ],
      "Shoulder Bags": [
        ["https://images.unsplash.com/photo-1605733513597-a8f8341084e6", "https://images.unsplash.com/photo-1591561954555-607968c989ab"],
        ["https://images.unsplash.com/photo-1591561954557-26941169b49e", "https://images.unsplash.com/photo-1584917865442-de89df76afd3"],
        ["https://images.unsplash.com/photo-1590739293931-a0975667f140", "https://images.unsplash.com/photo-1606522754091-a3ea32ec1b2b"]
      ]
    };

    // Generate remaining products to reach 50
    for (let i = sampleProducts.length; i < 50; i++) {
      const category = categories[i % categories.length];
      const adjective = adjectives[i % adjectives.length];
      const style = styles[i % styles.length];
      const basePrice = 200 + (i * 25); // Varies price between products

      // Get images for this category
      const categoryImageSet = categoryImages[category];
      const images = categoryImageSet[i % categoryImageSet.length];

      sampleProducts.push({
        name: `${adjective} ${style} ${category.slice(0, -1)}`,
        description: `${style} ${category.slice(0, -1).toLowerCase()} crafted from premium leather. Features multiple compartments and signature hardware details.`,
        price: basePrice.toFixed(2),
        images,
        category,
        inStock: 5 + (i % 10) // Varies stock between 5-15
      });
    }

    sampleProducts.forEach(product => this.createProduct(product));
  }

  async getProducts(): Promise<Product[]> {
    return Array.from(this.products.values());
  }

  async getProduct(id: number): Promise<Product | undefined> {
    return this.products.get(id);
  }

  async createProduct(product: InsertProduct): Promise<Product> {
    const id = this.currentIds.products++;
    const newProduct = { ...product, id };
    this.products.set(id, newProduct);
    return newProduct;
  }

  async getCartItems(sessionId: string): Promise<CartItem[]> {
    return Array.from(this.cartItems.values()).filter(
      item => item.sessionId === sessionId
    );
  }

  async addCartItem(item: InsertCartItem): Promise<CartItem> {
    const id = this.currentIds.cartItems++;
    const newItem = { ...item, id };
    this.cartItems.set(id, newItem);
    return newItem;
  }

  async removeCartItem(id: number): Promise<void> {
    this.cartItems.delete(id);
  }

  async updateCartItemQuantity(id: number, quantity: number): Promise<CartItem> {
    const item = this.cartItems.get(id);
    if (!item) throw new Error("Cart item not found");

    const updatedItem = { ...item, quantity };
    this.cartItems.set(id, updatedItem);
    return updatedItem;
  }

  async addNewsletterSubscription(sub: InsertNewsletter): Promise<Newsletter> {
    const id = this.currentIds.newsletters++;
    const newSub = { ...sub, id, createdAt: new Date() };
    this.newsletters.set(id, newSub);
    return newSub;
  }

  async createContact(contact: InsertContact): Promise<Contact> {
    const id = this.currentIds.contacts++;
    const newContact = { ...contact, id, createdAt: new Date() };
    this.contacts.set(id, newContact);
    return newContact;
  }
}

export const storage = new MemStorage();